.CanvasToolbar_enabled__2bOtL{cursor:pointer;background:none}.CanvasToolbar_disabled__T9sX8{cursor:not-allowed;-webkit-filter:invert(95%) sepia(10%) saturate(657%) hue-rotate(184deg) brightness(92%) contrast(95%);filter:invert(95%) sepia(10%) saturate(657%) hue-rotate(184deg) brightness(92%) contrast(95%)}.CanvasToolbar_enabled__2bOtL:hover{-webkit-filter:invert(41%) sepia(62%) saturate(7158%) hue-rotate(344deg) brightness(101%) contrast(108%);filter:invert(41%) sepia(62%) saturate(7158%) hue-rotate(344deg) brightness(101%) contrast(108%)}.CanvasToolbar_invertx__2gc2O{-webkit-transform:scaleX(-1);transform:scaleX(-1)}:root{box-sizing:border-box}*,:after,:before{box-sizing:inherit}body{margin:0;background:transparent}
/*# sourceMappingURL=main.41cd645d.chunk.css.map */